@@ -1,18 +1,36 @@
+/* FloboPuyo
+ * Copyright (C) 2004
+ *   Florent Boudet        <flobo@ios-software.com>,
+ *   Jean-Christophe Hoelt <jeko@ios-software.com>,
+ *   Guillaume Borios      <gyom@ios-software.com>
+ *
+ * iOS Software <http://www.ios-software.com>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ *
+ */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include "PuyoGame.h"
 #include "audio.h"
-#include <SDL/SDL.h>
+#include "glSDL.h"
 
 
-static int fallingTable[PUYODIMX][PUYODIMX] = {
-{2},
-{1, 4},
-{0, 3, 5},
-{0, 2, 4, 5},
-{0, 1, 3, 4, 5},
-{0, 1, 2, 3, 4, 5}
-};
+static int fallingTable[PUYODIMX] = {0, 3, 1, 4, 2, 5};
 
 PuyoRandomSystem::PuyoRandomSystem()
 {
@@ -34,14 +52,18 @@ PuyoPuyo::PuyoPuyo(PuyoState state)
 {
     this->state = state;
     X = 0; Y = 0;
-    attachedObject = NULL;
+}
+
+bool PuyoPuyo::isFalling()
+{
+    return (state < PUYO_EMPTY);
 }
 
 PuyoState PuyoPuyo::getPuyoState()
 {
-  if (this != NULL)
-    return state;
-  return PUYO_EMPTY;
+    if (this != NULL)
+        return state;
+    return PUYO_EMPTY;
 }
 
 void PuyoPuyo::setPuyoState(PuyoState state)
@@ -72,24 +94,24 @@ void PuyoPuyo::setPuyoXY(int X, int Y)
   }
 }
 
-void PuyoPuyo::setAttachedObject(void *object)
+PuyoGame::PuyoGame(PuyoRandomSystem *attachedRandom,
+		   PuyoFactory *attachedFactory)
 {
-  if (this != NULL)
-    attachedObject = object;
+  this->attachedFactory = attachedFactory;
+  InitGame(attachedRandom);
 }
 
-void *PuyoPuyo::getAttachedObject()
-{
-  if (this != NULL)
-    return attachedObject;
-  return NULL;
-}
-
-
 PuyoGame::PuyoGame(PuyoRandomSystem *attachedRandom)
 {
+  attachedFactory = new PuyoDefaultFactory();
+  InitGame(attachedRandom);
+}
+
+void PuyoGame::InitGame(PuyoRandomSystem *attachedRandom)
+{
+  nbFalled = 0;
 	int i, j;
-    unmoveablePuyo = new PuyoPuyo(PUYO_UNMOVEABLE);
+    unmoveablePuyo = attachedFactory->createPuyo(PUYO_UNMOVEABLE);
 	for (i = 0 ; i < PUYODIMX ; i++) {
 		for (j = 0 ; j <= PUYODIMY ; j++) {
 			if (j == PUYODIMY)
@@ -106,6 +128,7 @@ PuyoGame::PuyoGame(PuyoRandomSystem *attachedRandom)
         endOfCycle = false;
 	gameRunning = true;
 	setFallingAtTop(true);
+  points = 0;
 }
 
 PuyoGame::~PuyoGame()
@@ -194,7 +217,9 @@ void PuyoGame::moveLeft()
 	}
 	if (((fallingY<0)&&(fallingX>0))||((getPuyoCellAt(fallingX-1, fallingY) <= PUYO_EMPTY)
 	 && (getPuyoCellAt(getFallingCompanionX()-1, getFallingCompanionY()) <= PUYO_EMPTY)))
-		fallingX--;
+        {
+            fallingX--;
+        }
     fallingPuyo->setPuyoXY(fallingX, fallingY);
     companionPuyo->setPuyoXY(getFallingCompanionX(), getFallingCompanionY());
 }
@@ -211,76 +236,58 @@ void PuyoGame::moveRight()
     companionPuyo->setPuyoXY(getFallingCompanionX(), getFallingCompanionY());
 }
 
-void PuyoGame::rotateLeft()
+void PuyoGame::rotate(bool left)
 {
     if (endOfCycle) {
-		return;
-	}
-    audio_sound_play(sound_fff);
-	unsigned char backupCompanion = fallingCompanion;
-	/*int backupX = fallingX;
-	int backupY = fallingY;*/
-	int newX, newY;
+        return;
+    }
+    unsigned char backupCompanion = fallingCompanion;
+    /*int backupX = fallingX;
+    int backupY = fallingY;*/
+    int newX, newY;
     bool moved = true;
-	fallingCompanion = (unsigned char)(4 + fallingCompanion - 1) % 4;
-	unsigned char newCompanion = fallingCompanion;
-	int newCompanionX = getFallingCompanionX();
-	int newCompanionY = getFallingCompanionY();
-	fallingCompanion = backupCompanion;
-	if (((fallingY<0)&&(fallingX>0)&&(fallingX<PUYODIMX-1))||(getPuyoCellAt(newCompanionX, newCompanionY) > PUYO_EMPTY)) {
-		newX = fallingX + (fallingX - newCompanionX);
-		newY = fallingY + (fallingY  - newCompanionY);
-		if ((fallingY >= 0) && (getPuyoCellAt(newX, newY) > PUYO_EMPTY)) {
-      moved = false;
-		}
-		else {
-			fallingCompanion = newCompanion;
-			fallingX = newX;
-			fallingY = newY;
-		}
-	}
-	else
-		fallingCompanion = newCompanion;
-    fallingPuyo->setPuyoXY(fallingX, fallingY);
+    fallingCompanion = (unsigned char)(fallingCompanion + (left?3:1)) % 4;
+    unsigned char newCompanion = fallingCompanion;
+    int newCompanionX = getFallingCompanionX();
+    int newCompanionY = getFallingCompanionY();
+    fallingCompanion = backupCompanion;
+    if (getPuyoCellAt(newCompanionX, newCompanionY) > PUYO_EMPTY) {
+        //if (((fallingY<0)&&(fallingX>0)&&(fallingX<PUYODIMX-1))||(getPuyoCellAt(newCompanionX, newCompanionY) > PUYO_EMPTY)) {
+        newX = fallingX + (fallingX - newCompanionX);
+        newY = fallingY + (fallingY - newCompanionY);
+        //        if ((fallingY >= 0) && (getPuyoCellAt(newX, newY) > PUYO_EMPTY)) {
+        if (getPuyoCellAt(newX, newY) > PUYO_EMPTY) {
+            moved = false;
+        }
+        else {
+            fallingCompanion = newCompanion;
+            fallingX = newX;
+            if (fallingY != newY)
+            {
+                semiMove = 0;
+                fallingY = newY;
+            }
+            fallingPuyo->setPuyoXY(fallingX, fallingY);
+        }
+        }
+    else
+        fallingCompanion = newCompanion;
+    
     companionPuyo->setPuyoXY(getFallingCompanionX(), getFallingCompanionY());
     
     if ((delegate != NULL) && (moved))
-        delegate->companionDidTurn(companionPuyo, fallingCompanion, false);
+        delegate->companionDidTurn(companionPuyo, fallingCompanion, !left);
+    
+}
+
+void PuyoGame::rotateLeft()
+{
+    rotate(true);
 }
 
 void PuyoGame::rotateRight()
 {
-    if (endOfCycle) {
-		return;
-	}
-    audio_sound_play(sound_fff);
-	unsigned char backupCompanion = fallingCompanion;
-	int backupX = fallingX;
-	int backupY = fallingY;
-    bool moved = true;
-	fallingCompanion = (unsigned char)(fallingCompanion + 1) % 4;
-	unsigned char newCompanion = fallingCompanion;
-	int newCompanionX = getFallingCompanionX();
-	int newCompanionY = getFallingCompanionY();
-	fallingCompanion = backupCompanion;
-	if (((fallingY<0)&&(fallingX>0)&&(fallingX<PUYODIMX-1))||(getPuyoCellAt(newCompanionX, newCompanionY) > PUYO_EMPTY)) {
-		fallingX += (fallingX - newCompanionX);
-		fallingY += (fallingY  - newCompanionY);
-		if ((fallingY >= 0) && (getPuyoCellAt(fallingX, fallingY) > PUYO_EMPTY)) {
-			fallingX = backupX;
-			fallingY = backupY;
-            moved = false;
-		}
-		else
-			fallingCompanion = newCompanion;
-	}
-	else
-		fallingCompanion = newCompanion;
-    fallingPuyo->setPuyoXY(fallingX, fallingY);
-    companionPuyo->setPuyoXY(getFallingCompanionX(), getFallingCompanionY());
-    
-    if ((delegate != NULL) && (moved))
-        delegate->companionDidTurn(companionPuyo, fallingCompanion, true);
+    rotate(false);
 }
 
 PuyoState PuyoGame::getNextFalling()
@@ -324,24 +331,32 @@ void PuyoGame::setPuyoAt(int X, int Y, PuyoPuyo *newPuyo)
 
 void PuyoGame::dropNeutrals()
 {
-    while (neutralPuyos > 0) {
+  if (neutralPuyos < 0) {
+    extern int gameLevel;
+    points -= gameLevel * neutralPuyos * 1000;
+  }
+
+    int idNeutral = 0;
+    while (neutralPuyos > 0)
+    {
       int cycleNeutral;
       if (neutralPuyos >=  PUYODIMX)
         cycleNeutral = PUYODIMX;
       else
         cycleNeutral = neutralPuyos;
-      for (int i = 0 ; i < cycleNeutral ; i++) {
-        int posX = fallingTable[cycleNeutral-1][i];
+      for (int i = 0 ; i < cycleNeutral ; i++)
+      {
+        int posX = fallingTable[(nbFalled++) % PUYODIMX];
         int posY = getFallY(posX, 2);
         neutralPuyos -= 1;
         if (getPuyoCellAt(posX, posY) != PUYO_EMPTY)
             continue;
         // Creating a new neutral puyo
-        PuyoPuyo *newNeutral = new PuyoPuyo(PUYO_NEUTRAL);
+        PuyoPuyo *newNeutral = attachedFactory->createPuyo(PUYO_NEUTRAL);
         puyoVector.addElement(newNeutral);
         setPuyoAt(posX, posY, newNeutral);
         if (delegate != NULL)
-          delegate->gameDidAddNeutral(newNeutral);
+          delegate->gameDidAddNeutral(newNeutral, idNeutral++);
       }
     }
     neutralPuyos = 0;
@@ -354,34 +369,27 @@ void PuyoGame::setFallingAtTop(bool gameConstruction)
 
   if (!gameConstruction) {
     dropNeutrals();
-    //for (int x=0;x<PUYODIMX;++x) {
-        if (getPuyoCellAt(PUYODIMX/2, 1) != PUYO_EMPTY) {
+        if (getPuyoCellAt((PUYODIMX-1)/2, 1) != PUYO_EMPTY) {
             gameRunning = false;
             if (delegate != NULL)
                 delegate->gameLost();
             return;
         }
-    //}
-//    if ((getPuyoCellAt(PUYODIMX/2, 0) != PUYO_EMPTY)
-//        || (getPuyoCellAt(PUYODIMX/2, 1) != PUYO_EMPTY)) {
-//      gameRunning = false;
-//      if (delegate != NULL)
-//        delegate->gameLost();
-//    }
   }
 
   // Creating the new falling puyo and its companion
-  fallingX = PUYODIMX/2;
+  fallingX = (PUYODIMX-1)/2;
   fallingY = 1;
-  fallingCompanion = 0;
-  fallingPuyo = new PuyoPuyo(attachedRandom->getPuyoForSequence(sequenceNr++));
-  companionPuyo = new PuyoPuyo(attachedRandom->getPuyoForSequence(sequenceNr++));
+  fallingCompanion = 2;
+  fallingPuyo = attachedFactory->createPuyo(attachedRandom->getPuyoForSequence(sequenceNr++));
+  companionPuyo = attachedFactory->createPuyo(attachedRandom->getPuyoForSequence(sequenceNr++));
   fallingPuyo->setPuyoXY(fallingX, fallingY);
   companionPuyo->setPuyoXY(getFallingCompanionX(), getFallingCompanionY());
   puyoVector.addElement(fallingPuyo);
   puyoVector.addElement(companionPuyo);
 
   endOfCycle = false;
+  semiMove = 0;
   phase = 0;
 }
 
@@ -514,14 +522,14 @@ void PuyoGame::deleteMarkedPuyosAt(int X, int Y)
 	PuyoState currentPuyo = getPuyoCellAt(X, Y);
   puyoVector.removeElement(getPuyoAt(X, Y));
   if (getPuyoAt(X,Y) == companionPuyo) {
-    delete (getPuyoAt(X, Y));
+    attachedFactory->deletePuyo(getPuyoAt(X, Y));
     companionPuyo = NULL;
   }
   else if (getPuyoAt(X,Y) == fallingPuyo) {
-    delete (getPuyoAt(X, Y));
+    attachedFactory->deletePuyo(getPuyoAt(X, Y));
     fallingPuyo = NULL;
   } else {
-    delete (getPuyoAt(X, Y));
+    attachedFactory->deletePuyo(getPuyoAt(X, Y));
   }
   setPuyoAt(X, Y, NULL);
 	if (getPuyoCellAt(X-1, Y) == currentPuyo)
@@ -534,22 +542,22 @@ void PuyoGame::deleteMarkedPuyosAt(int X, int Y)
 		deleteMarkedPuyosAt(X, Y+1);
     if (getPuyoCellAt(X-1, Y) == PUYO_NEUTRAL) {
         puyoVector.removeElement(getPuyoAt(X-1, Y));
-        delete (getPuyoAt(X-1, Y));
+        attachedFactory->deletePuyo(getPuyoAt(X-1, Y));
         setPuyoAt(X-1, Y, NULL);
     }
     if (getPuyoCellAt(X+1, Y) == PUYO_NEUTRAL) {
         puyoVector.removeElement(getPuyoAt(X+1, Y));
-        delete (getPuyoAt(X+1, Y));
+        attachedFactory->deletePuyo(getPuyoAt(X+1, Y));
         setPuyoAt(X+1, Y, NULL);
     }
     if (getPuyoCellAt(X, Y-1) == PUYO_NEUTRAL) {
         puyoVector.removeElement(getPuyoAt(X, Y-1));
-        delete (getPuyoAt(X, Y-1));
+        attachedFactory->deletePuyo(getPuyoAt(X, Y-1));
         setPuyoAt(X, Y-1, NULL);
     }
     if (getPuyoCellAt(X, Y+1) == PUYO_NEUTRAL) {
         puyoVector.removeElement(getPuyoAt(X, Y+1));
-        delete (getPuyoAt(X, Y+1));
+        attachedFactory->deletePuyo(getPuyoAt(X, Y+1));
         setPuyoAt(X, Y+1, NULL);
     }
 }
@@ -581,53 +589,73 @@ int PuyoGame::removePuyos()
 			}
 		}
 	}
-
+    
 	/* Next we make the other puyos fall */
-	int fallPuyo;
-	do {
-		fallPuyo = 0;
-		for (int i = 0 ; i < PUYODIMX ; i++) {
-			for (int j = 0 ; j <= PUYODIMY ; j++) {
-				PuyoPuyo *currentPuyo = getPuyoAt(i, j);
-                PuyoState currentPuyoState = getPuyoCellAt(i, j);
-				if ((currentPuyoState >= PUYO_BLUE) && (currentPuyoState <= PUYO_NEUTRAL)) {
-					if (getPuyoCellAt(i, j+1) == PUYO_EMPTY) {
-						setPuyoAt(i, j, NULL);
-						setPuyoAt(i, j+1, currentPuyo);
-                        if (delegate != NULL)
-                            delegate->puyoDidFall(currentPuyo, i, j);
-						fallPuyo++;
-					}
-				}
-			}
-		} 
-	} while (fallPuyo > 0);
+    for (int i = 0 ; i < PUYODIMX ; i++) {
+        for (int j = PUYODIMY - 1 ; j > 0 ; j--) {
+            PuyoState currentPuyoState = getPuyoCellAt(i, j);
+            if ((currentPuyoState >= PUYO_BLUE) && (currentPuyoState <= PUYO_NEUTRAL)) {
+                int newJ = getFallY(i, j);
+                if (newJ != j) {
+                    PuyoPuyo *currentPuyo = getPuyoAt(i, j);
+                    setPuyoAt(i, j, NULL);
+                    setPuyoAt(i, newJ, currentPuyo);
+                    if (delegate != NULL) {
+                        delegate->puyoDidFall(currentPuyo, i, j);
+                    }
+                }
+            }
+        }
+    }
+    
 	return globalRemovedPuyos;
 }
 
 
 void PuyoGame::notifyReductions()
 {
-    for (int i = 0 ; i < PUYODIMX ; i++) {
-		for (int j = 0 ; j <= PUYODIMY ; j++) {
-			PuyoState currentPuyo = getPuyoCellAt(i, j);
-			if ((currentPuyo >= PUYO_BLUE) && (currentPuyo <= PUYO_YELLOW)) {
-				int removedPuyos = 0;
-				markPuyoAt(i, j, PUYO_MARKED, false);
-				for (int u = 0 ; u < PUYODIMX ; u++) {
-					for (int v = 0 ; v <= PUYODIMY ; v++) {
-						if (getPuyoCellAt(u, v) == PUYO_MARKED) {
-                            removedPuyos++;
-						}
-					}
-				}
-                if (removedPuyos >= 4) {
-                    markPuyoAt(i, j, currentPuyo, false);
-                    if (delegate != NULL)
-                        delegate->puyoWillVanish(getPuyoAt(i, j));
+    IosVector removedPuyos;
+    
+    // Clearing every puyo's flag
+    for (int i = 0, j = getPuyoCount() ; i < j ; i++) {
+        getPuyoAtIndex(i)->unsetFlag();
+    }
+    // Search for groupped puyos
+    int puyoGroupNumber = 0;
+    for (int j = 0 ; j < PUYODIMY ; j++) {
+		for (int i = 0 ; i <= PUYODIMX ; i++) {
+            PuyoPuyo *puyoToMark = getPuyoAt(i, j);
+            // If the puyo exists and is not flagged, then
+            if ((puyoToMark != NULL) && (! puyoToMark->getFlag())) {
+                PuyoState initialPuyoState = puyoToMark->getPuyoState();
+                // I really would have liked to skip this stupid test
+                if ((initialPuyoState >= PUYO_BLUE) && (initialPuyoState <= PUYO_YELLOW)) {
+                    markPuyoAt(i, j, PUYO_MARKED, false);
+                    
+                    // Collecting every marked puyo in a vector
+                    removedPuyos.removeAllElements();
+                    for (int u = 0 ; u < PUYODIMX ; u++) {
+                        for (int v = 0 ; v <= PUYODIMY ; v++) {
+                            PuyoPuyo *markedPuyo = getPuyoAt(u, v);
+                            if (markedPuyo->getPuyoState() == PUYO_MARKED) {
+                                // mark the puyo so we wont'do the job twice
+                                markedPuyo->setFlag();
+                                removedPuyos.addElement(markedPuyo);
+                            }
+                        }
+                    }
+                    
+                    
+                    // If there is more than 4 puyo in the group, let's notify it
+                    if (removedPuyos.getSize() >= 4) {
+                        markPuyoAt(i, j, initialPuyoState, false);
+                        if (delegate != NULL)
+                            delegate->puyoWillVanish(removedPuyos, puyoGroupNumber++, phase);
+                    }
+                    else {
+                        markPuyoAt(i, j, initialPuyoState, false);
+                    }
                 }
-                else
-                    markPuyoAt(i, j, currentPuyo, false);
 			}
 		}
 	}
@@ -637,23 +665,21 @@ void PuyoGame::cycleEnding()
 {
   static int cmpt = 0;
 	int score = removePuyos();
+    
 	if (score >= 4) {
+    #ifdef DESACTIVE
     audio_sound_play(sound_splash[phase>7?7:phase]);
-    if (phase>=2) {
-      audio_sound_play(sound_yahoohoo3[(int)((float)NB_YAHOOHOO3 * random()/(RAND_MAX+1.0))]);
-    }
-    if (phase==1) {
-      audio_sound_play(sound_yahoohoo2[(int)((float)NB_YAHOOHOO2 * random()/(RAND_MAX+1.0))]);
-    }
-    else {
-      audio_sound_play(sound_yahoohoo1[(int)((float)NB_YAHOOHOO1 * random()/(RAND_MAX+1.0))]);
-    }
+    #endif
     score -= 3;
 	  if (phase > 0) {
 	    neutralPuyos -= PUYODIMX;
     }
 	  phase++;
 	}
+  
+  extern int gameLevel;
+  points += gameLevel * 100 + gameLevel * (phase>0?phase-1:0) * 5000;
+  
 	neutralPuyos -= score;
 	if (score == 0)
 		setFallingAtTop();
